/*
 * NCURSUI.H -- NCURSES USER INTERFACE UTILS
 * 
 * Copyright (C) 1998 by Ralph F. Siemsen
 * Released under the GNU public license
 * 
 * Identification:
 *   $Id: ncursui.h,v 1.6 1999/04/08 17:45:19 jasona Exp $
 */

#ifndef RFS_NCURSUI_H
#define RFS_NCURSUI_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <unistd.h>
#include <curses.h>
#include <panel.h>

enum
{
	HOSTNAME,
	DOMAIN,
	EXTERNAL_IPADDR,
	EXTERNAL_NETMASK,
	INTERNAL_IPADDR,
	INTERNAL_NETMASK,
	GATEWAY,
	NAMESERVER,
	INTERVAL,
	LOADBAL,
	TIMEZONE,
	TIMEDATE,
	NFSSERV,
	NFSBIN,
	NFSDOC,
	NETADDR,

	SERVERNAME,

	V_HOSTNAME,
	V_DOMAIN,
	V_IPADDR,
	V_PORT,
	V_DOCROOT,
	V_CGIROOT,
	V_INCLUDES,
	V_LOADBAL,
	V_NODELIST,

	ITEM_MAX
};

#define REDIR ">/dev/null 2>&1"

extern int LINES, COLS;

struct Menu;			// defined later

/* A Window is a rectangular area of the screen possibly with a border
   and a title.  Clipped text may be displayed inside a window at a 
   specified position with the Print() routine. */

struct Window {
  static int window_count;
  int left, top;
  int width, height;
  WINDOW *win;
  PANEL *pan;
  int visible;
  char *title;

  Window (int Left, int Top, int Width, int Height);
  ~Window ();
  inline void Update ();
  void Box (int Draw, int Left, int Top, int Width, int Height);
  void Border (int Draw);
  void Title (char *Title);
  void Print (int Col, int Line, char *Text);
  void Help (char *Msg);
  int Select (Menu *Menu, int Pre);
  int DrawMenu (Menu *menu);
};


/* A Field is an array of characters on the screen. */

#define FIELD_MAX 128

struct Field {
  Window *win;
  int row;
  int col;
  char value[FIELD_MAX];
  int width;
  int id;
  int cursor;
  int changed;
  Field (Window *Win, int Row, int Col, int Width, int Id);
  void Help (Window *HelpWin, int MsgNum);
  virtual void Display (int Highlight) = 0;
  virtual int Edit (int Ch) = 0;
  virtual int Validate (void) = 0;
  int ShowError (int Row, int Col, int MsgNum);
};

struct TextField : public Field {
  int scroll;		// Amount by which value is scrolled left
  TextField (Window *Win, int Row, int Col, int Width, int Id);
  void Display (int Highlight);
  int Edit (int Ch);
  int Validate (void);
};

struct NumberField : public TextField {
  NumberField (Window *Win, int Row, int Col, int Width, int Id);
  int Edit (int Ch);
  int Validate (void);
};

struct PathField : public Field {
  int scroll;		// Amount by which value is scrolled left
  PathField (Window *Win, int Row, int Col, int Width, int Id);
  void Display (int Highlight);
  int Edit (int Ch);
  int Validate (void);
};

struct NumberListField : public Field {
  int scroll;		// Amount by which value is scrolled left
  NumberListField (Window *Win, int Row, int Col, int Width, int Id);
  void Display (int Highlight);
  int Edit (int Ch);
  int Validate (void);
};

struct HostNameField : public TextField {
  HostNameField (Window *Win, int Row, int Col, int Width, int Id);
  int Edit (int Ch);
  int Validate (void);
};

struct IPField : public TextField {
  IPField (Window *Win, int Row, int Col, int Width, int Id);
  int Edit (int Ch);
  int Validate (void);
};

struct MultiIPField : public TextField {
  MultiIPField (Window *Win, int Row, int Col, int Width, int Id);
  int Edit (int Ch);
  int Validate (void);
};

/* Selection fields */

struct SelectField : public Field {
  int choice;
  char **options;
  int count;
  SelectField (Window *Win, int Row, int Col, 
    int Width, char **Options, int Id);
  void Display (int Highlight);
  int Edit (int Ch);
  int Validate (void);
};

struct SpecialSelectField : public SelectField {
  typedef void (*fptr)();
  fptr action;
  SpecialSelectField (Window *Win, int Row, int Col, 
    int Width, fptr Action, char **Options, int Id);
  int Edit (int Ch);
  int Validate (void);
};

/* Time entry field */

struct TimeField : public Field {
  time_t now;
  struct tm tm;
  time_t offset;
  TimeField (Window *Win, int Row, int Col, int Id);
  void Display (int Highlight);
  int Edit (int Ch);
  int Validate (void);
  void Adjust (int Field, int Amount);
};

/* Forms are collections of fields, each of which has a label, an x,y
   position and a maximum display width.  A field can be displayed in
   highlight or normal.  Each field is attached to a Window.  */

struct Form {
  int count;		// Number of items in this form (computed)
  int selected;		// Which item currently has the input focus
  Field **items;	// null-terminated list of member items
  Window *helpwin;	// Pointer to window in which to display help
  Form (Field **forms, Window *HelpWin);
  ~Form (void);
  int Edit (int Ch);
  void Render (void);
};

/*** File handling ***/

struct File {
  FILE *fh;
  File (char *name);
  ~File (void);
  char *ReadParam (char *name);
  int QueryWebFront (void);
};

struct FWrite {
  FILE *fh;
  FWrite (char *name);
  ~FWrite (void);
  void Printf (char *str);
};

/*** Menus ***/

#define MITEM(args...) ((struct MenuItem){args})

struct MenuItem			/* ONE SELECTION ON A MENU */
  {
    char *Prefix;		/* Stuff before the highlight 	*/
    char Hotkey;		/* The highlighed letter 	*/
    char *Suffix;		/* Stuff after the highlight 	*/
    void *Code;			/* Future expansion 		*/
    void *Data;			/* Future expansion 		*/
  };

struct Menu			/* AN ENTIRE MENU STRUCTURE */
  {
    int Lines, Cols;		/* Physical dimensions 		*/
    int Gap;			/* Spacing between columns 	*/
    struct MenuItem **Item;	/* MenuItem fields follow 	*/
  };

#endif // RFS_NCURSUI_H
